@@ -12,9 +12,6 @@
 #include "recursor_cache.hh"
 #include "syncres.hh"
 #include "negcache.hh"
-#include <boost/function.hpp>
-#include <boost/optional.hpp>
-#include <boost/tuple/tuple.hpp>
 #include <boost/format.hpp>
 #include <boost/algorithm/string.hpp>
 
@@ -40,6 +37,7 @@
 #include "namespaces.hh"
 #include "rec-taskqueue.hh"
 #include "rec-tcpout.hh"
+#include "rec-main.hh"
 
 std::pair<std::string, std::string> PrefixDashNumberCompare::prefixAndTrailingNum(const std::string& a)
 {
@@ -157,9 +155,9 @@ std::atomic<unsigned long>* getDynMetric(const std::string& str, const std::stri
   return ret.d_ptr;
 }
 
-static boost::optional<uint64_t> get(const string& name)
+static std::optional<uint64_t> get(const string& name)
 {
-  boost::optional<uint64_t> ret;
+  std::optional<uint64_t> ret;
 
   if (d_get32bitpointers.count(name))
     return *d_get32bitpointers.find(name)->second;
@@ -187,7 +185,7 @@ static boost::optional<uint64_t> get(const string& name)
   return ret;
 }
 
-boost::optional<uint64_t> getStatByName(const std::string& name)
+std::optional<uint64_t> getStatByName(const std::string& name)
 {
   return get(name);
 }
@@ -247,7 +245,7 @@ static string doGet(T begin, T end)
   string ret;
 
   for (T i = begin; i != end; ++i) {
-    boost::optional<uint64_t> num = get(*i);
+    std::optional<uint64_t> num = get(*i);
     if (num)
       ret += std::to_string(*num) + "\n";
     else
@@ -356,7 +354,7 @@ static uint64_t dumpAggressiveNSECCache(int fd)
 
 static uint64_t* pleaseDump(int fd)
 {
-  return new uint64_t(t_packetCache->doDump(fd));
+  return new uint64_t(t_packetCache ? t_packetCache->doDump(fd) : 0);
 }
 
 static uint64_t* pleaseDumpEDNSMap(int fd)
@@ -379,13 +377,23 @@ static uint64_t* pleaseDumpFailedServers(int fd)
   return new uint64_t(SyncRes::doDumpFailedServers(fd));
 }
 
+static uint64_t* pleaseDumpSavedParentNSSets(int fd)
+{
+  return new uint64_t(SyncRes::doDumpSavedParentNSSets(fd));
+}
+
 static uint64_t* pleaseDumpNonResolvingNS(int fd)
 {
   return new uint64_t(SyncRes::doDumpNonResolvingNS(fd));
 }
 
+static uint64_t* pleaseDumpDoTProbeMap(int fd)
+{
+  return new uint64_t(SyncRes::doDumpDoTProbeMap(fd));
+}
+
 // Generic dump to file command
-static RecursorControlChannel::Answer doDumpToFile(int s, uint64_t* (*function)(int s), const string& name)
+static RecursorControlChannel::Answer doDumpToFile(int s, uint64_t* (*function)(int s), const string& name, bool threads = true)
 {
   auto fdw = getfd(s);
 
@@ -395,8 +403,15 @@ static RecursorControlChannel::Answer doDumpToFile(int s, uint64_t* (*function)(
 
   uint64_t total = 0;
   try {
-    int fd = fdw;
-    total = broadcastAccFunction<uint64_t>([function, fd] { return function(fd); });
+    if (threads) {
+      int fd = fdw;
+      total = broadcastAccFunction<uint64_t>([function, fd] { return function(fd); });
+    }
+    else {
+      auto ret = function(fdw);
+      total = *ret;
+      delete ret;
+    }
   }
   catch (std::exception& e) {
     return {1, name + ": error dumping data: " + string(e.what()) + "\n"};
@@ -804,7 +819,7 @@ static string setMinimumTTL(T begin, T end)
   if (end - begin != 1)
     return "Need to supply new minimum TTL number\n";
   try {
-    SyncRes::s_minimumTTL = pdns_stou(*begin);
+    pdns::checked_stoi_into(SyncRes::s_minimumTTL, *begin);
     return "New minimum TTL: " + std::to_string(SyncRes::s_minimumTTL) + "\n";
   }
   catch (const std::exception& e) {
@@ -818,7 +833,7 @@ static string setMinimumECSTTL(T begin, T end)
   if (end - begin != 1)
     return "Need to supply new ECS minimum TTL number\n";
   try {
-    SyncRes::s_minimumECSTTL = pdns_stou(*begin);
+    pdns::checked_stoi_into(SyncRes::s_minimumECSTTL, *begin);
     return "New minimum ECS TTL: " + std::to_string(SyncRes::s_minimumECSTTL) + "\n";
   }
   catch (const std::exception& e) {
@@ -832,7 +847,7 @@ static string setMaxCacheEntries(T begin, T end)
   if (end - begin != 1)
     return "Need to supply new cache size\n";
   try {
-    g_maxCacheEntries = pdns_stou(*begin);
+    g_maxCacheEntries = pdns::checked_stoi<uint32_t>(*begin);
     return "New max cache entries: " + std::to_string(g_maxCacheEntries) + "\n";
   }
   catch (const std::exception& e) {
@@ -845,8 +860,11 @@ static string setMaxPacketCacheEntries(T begin, T end)
 {
   if (end - begin != 1)
     return "Need to supply new packet cache size\n";
+  if (::arg().mustDo("disable-packetcache")) {
+    return "Packet cache is disabled\n";
+  }
   try {
-    g_maxPacketCacheEntries = pdns_stou(*begin);
+    g_maxPacketCacheEntries = pdns::checked_stoi<uint32_t>(*begin);
     return "New max packetcache entries: " + std::to_string(g_maxPacketCacheEntries) + "\n";
   }
   catch (const std::exception& e) {
@@ -963,32 +981,6 @@ static uint64_t getNegCacheSize()
   return g_negCache->size();
 }
 
-static uint64_t* pleaseGetFailedHostsSize()
-{
-  uint64_t tmp = (SyncRes::getThrottledServersSize());
-  return new uint64_t(tmp);
-}
-
-static uint64_t getFailedHostsSize()
-{
-  return broadcastAccFunction<uint64_t>(pleaseGetFailedHostsSize);
-}
-
-uint64_t* pleaseGetNsSpeedsSize()
-{
-  return new uint64_t(SyncRes::getNSSpeedsSize());
-}
-
-static uint64_t getNsSpeedsSize()
-{
-  return broadcastAccFunction<uint64_t>(pleaseGetNsSpeedsSize);
-}
-
-uint64_t* pleaseGetFailedServersSize()
-{
-  return new uint64_t(SyncRes::getFailedServersSize());
-}
-
 uint64_t* pleaseGetEDNSStatusesSize()
 {
   return new uint64_t(SyncRes::getEDNSStatusesSize());
@@ -1126,7 +1118,9 @@ static StatsMap toCPUStatsMap(const string& name)
 {
   const string pbasename = getPrometheusName(name);
   StatsMap entries;
-  for (unsigned int n = 0; n < g_numThreads; ++n) {
+  // Only distr and worker threads, I think we should revisit this as we now not only have the handler thread but also
+  // taskThread(s).
+  for (unsigned int n = 0; n < RecThreadInfo::numDistributors() + RecThreadInfo::numWorkers(); ++n) {
     uint64_t tm = doGetThreadCPUMsec(n);
     std::string pname = pbasename + "{thread=\"" + std::to_string(n) + "\"}";
     entries.emplace(name + "-thread-" + std::to_string(n), StatsMapEntry{pname, std::to_string(tm)});
@@ -1159,8 +1153,6 @@ static StatsMap toRPZStatsMap(const string& name, LockGuarded<std::unordered_map
   return entries;
 }
 
-extern ResponseStats g_rs;
-
 static void registerAllStats1()
 {
   addGetStat("questions", &g_stats.qcounter);
@@ -1250,8 +1242,9 @@ static void registerAllStats1()
   addGetStat("negcache-entries", getNegCacheSize);
   addGetStat("throttle-entries", getThrottleSize);
 
-  addGetStat("nsspeeds-entries", getNsSpeedsSize);
-  addGetStat("failed-host-entries", getFailedHostsSize);
+  addGetStat("nsspeeds-entries", SyncRes::getNSSpeedsSize);
+  addGetStat("failed-host-entries", SyncRes::getFailedServersSize);
+  addGetStat("non-resolving-nameserver-entries", SyncRes::getNonResolvingNSSize);
 
   addGetStat("concurrent-queries", getConcurrentQueries);
   addGetStat("security-status", &g_security_status);
@@ -1444,12 +1437,10 @@ void registerAllStats()
 void doExitGeneric(bool nicely)
 {
   g_log << Logger::Error << "Exiting on user request" << endl;
-  extern RecursorControlChannel s_rcc;
-  s_rcc.~RecursorControlChannel();
+  g_rcc.~RecursorControlChannel();
 
-  extern string s_pidfname;
-  if (!s_pidfname.empty())
-    unlink(s_pidfname.c_str()); // we can at least try..
+  if (!g_pidfname.empty())
+    unlink(g_pidfname.c_str()); // we can at least try..
   if (nicely) {
     RecursorControlChannel::stop = true;
   }
@@ -1506,8 +1497,8 @@ vector<pair<DNSName, uint16_t>>* pleaseGetBogusQueryRing()
   return ret;
 }
 
-typedef boost::function<vector<ComboAddress>*()> pleaseremotefunc_t;
-typedef boost::function<vector<pair<DNSName, uint16_t>>*()> pleasequeryfunc_t;
+typedef std::function<vector<ComboAddress>*()> pleaseremotefunc_t;
+typedef std::function<vector<pair<DNSName, uint16_t>>*()> pleasequeryfunc_t;
 
 vector<ComboAddress>* pleaseGetRemotes()
 {
@@ -1641,7 +1632,7 @@ static DNSName nopFilter(const DNSName& name)
   return name;
 }
 
-static string doGenericTopQueries(pleasequeryfunc_t func, boost::function<DNSName(const DNSName&)> filter = nopFilter)
+static string doGenericTopQueries(pleasequeryfunc_t func, std::function<DNSName(const DNSName&)> filter = nopFilter)
 {
   typedef pair<DNSName, uint16_t> query_t;
   typedef map<query_t, int> counts_t;
@@ -1678,7 +1669,7 @@ static string doGenericTopQueries(pleasequeryfunc_t func, boost::function<DNSNam
 
 static string* nopFunction()
 {
-  return new string("pong\n");
+  return new string("pong " + RecThreadInfo::self().getName() + '\n');
 }
 
 static string getDontThrottleNames()
@@ -1877,7 +1868,7 @@ static string setEventTracing(T begin, T end)
     return "No event trace enabled value specified\n";
   }
   try {
-    SyncRes::s_event_trace_enabled = pdns_stou(*begin);
+    pdns::checked_stoi_into(SyncRes::s_event_trace_enabled, *begin);
     return "New event trace enabled value: " + std::to_string(SyncRes::s_event_trace_enabled) + "\n";
   }
   catch (const std::exception& e) {
@@ -1885,6 +1876,12 @@ static string setEventTracing(T begin, T end)
   }
 }
 
+static void* pleaseSupplantProxyMapping(std::shared_ptr<ProxyMapping> pm)
+{
+  t_proxyMapping = pm;
+  return nullptr;
+}
+
 RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const string& question, RecursorControlParser::func_t** command)
 {
   *command = nop;
@@ -1912,10 +1909,13 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
             "clear-nta [DOMAIN]...            Clear the Negative Trust Anchor for DOMAINs, if no DOMAIN is specified, remove all\n"
             "clear-ta [DOMAIN]...             Clear the Trust Anchor for DOMAINs\n"
             "dump-cache <filename>            dump cache contents to the named file\n"
+            "dump-dot-probe-map <filename>    dump the contents of the DoT probe map to the named file\n"
             "dump-edns [status] <filename>    dump EDNS status to the named file\n"
             "dump-failedservers <filename>    dump the failed servers to the named file\n"
             "dump-non-resolving <filename>    dump non-resolving nameservers addresses to the named file\n"
             "dump-nsspeeds <filename>         dump nsspeeds statistics to the named file\n"
+            "dump-saved-parent-ns-sets <filename>\n"
+            "                                 dump saved parent ns sets that were successfully used as fallback\n"
             "dump-rpz <zone name> <filename>  dump the content of a RPZ zone to the named file\n"
             "dump-throttlemap <filename>      dump the contents of the throttle map to the named file\n"
             "get [key1] [key2] ..             get specific statistics\n"
@@ -1983,14 +1983,20 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
   if (cmd == "dump-cache") {
     return doDumpCache(s);
   }
+  if (cmd == "dump-dot-probe-map") {
+    return doDumpToFile(s, pleaseDumpDoTProbeMap, cmd, false);
+  }
   if (cmd == "dump-ednsstatus" || cmd == "dump-edns") {
     return doDumpToFile(s, pleaseDumpEDNSMap, cmd);
   }
   if (cmd == "dump-nsspeeds") {
-    return doDumpToFile(s, pleaseDumpNSSpeeds, cmd);
+    return doDumpToFile(s, pleaseDumpNSSpeeds, cmd, false);
   }
   if (cmd == "dump-failedservers") {
-    return doDumpToFile(s, pleaseDumpFailedServers, cmd);
+    return doDumpToFile(s, pleaseDumpFailedServers, cmd, false);
+  }
+  if (cmd == "dump-saved-parent-ns-sets") {
+    return doDumpToFile(s, pleaseDumpSavedParentNSSets, cmd, false);
   }
   if (cmd == "dump-rpz") {
     return doDumpRPZ(s, begin, end);
@@ -1999,7 +2005,7 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
     return doDumpToFile(s, pleaseDumpThrottleMap, cmd);
   }
   if (cmd == "dump-non-resolving") {
-    return doDumpToFile(s, pleaseDumpNonResolvingNS, cmd);
+    return doDumpToFile(s, pleaseDumpNonResolvingNS, cmd, false);
   }
   if (cmd == "wipe-cache" || cmd == "flushname") {
     return {0, doWipeCache(begin, end, 0xffff)};
@@ -2024,8 +2030,11 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
 
     try {
       luaConfigDelayedThreads delayedLuaThreads;
-      loadRecursorLuaConfig(::arg()["lua-config-file"], delayedLuaThreads);
+      ProxyMapping proxyMapping;
+      loadRecursorLuaConfig(::arg()["lua-config-file"], delayedLuaThreads, proxyMapping);
       startLuaConfigDelayedThreads(delayedLuaThreads, g_luaconfs.getCopy().generation);
+      std::shared_ptr<ProxyMapping> ptr = proxyMapping.empty() ? nullptr : std::make_shared<ProxyMapping>(proxyMapping);
+      broadcastFunction([=] { return pleaseSupplantProxyMapping(ptr); });
       g_log << Logger::Warning << "Reloaded Lua configuration file '" << ::arg()["lua-config-file"] << "', requested via control channel" << endl;
       return {0, "Reloaded Lua configuration file '" + ::arg()["lua-config-file"] + "'\n"};
     }
